@@ -9,7 +9,7 @@ from paco.models.logging import CloudWatchLogSources, CloudWatchLogSource, Metri
     CloudWatchLogGroups, CloudWatchLogGroup, CloudWatchLogSets, CloudWatchLogSet, CloudWatchLogging, \
     MetricTransformation
 from paco.models.exceptions import InvalidPacoFieldType, InvalidPacoProjectFile, UnusedPacoProjectField, \
-    TroposphereConversionError, InvalidPacoSchema, InvalidLocalPath
+    TroposphereConversionError, InvalidPacoSchema, InvalidLocalPath, InvalidPacoSub, InvalidPacoReference
 from paco.models.metrics import MonitorConfig, Metric, ec2core_builtin_metric, asg_builtin_metrics, \
     CloudWatchAlarm, SimpleCloudWatchAlarm, \
     AlarmSet, AlarmSets, AlarmNotifications, AlarmNotification, SNSTopics, Dimension, \
@@ -19,8 +19,8 @@ from paco.models.networks import NetworkEnvironment, Environment, EnvironmentDef
     PrivateHostedZone, SecurityGroup, IngressRule, EgressRule, NATGateways, VPNGateways, Segments, \
     VPCPeerings, SecurityGroupSets, SecurityGroups
 from paco.models.project import VersionControl, Project, Credentials, SharedState, PacoWorkBucket
-from paco.models.applications import Application, ResourceGroups, ResourceGroup, RDS, \
-    ASG, ECSASGConfiguration, \
+from paco.models.applications import Application, ResourceGroups, ResourceGroup, \
+    ASG, ECSASGConfiguration, SSHAccess, ElastiCacheRedis, \
     Resource, Resources, LBApplication, TargetGroups, TargetGroup, Listeners, Listener, DNS, PortProtocol, EC2, \
     S3Bucket, ApplicationS3Bucket, S3NotificationConfiguration, S3LambdaConfiguration, \
     S3StaticWebsiteHosting, S3StaticWebsiteHostingRedirectRequests, S3BucketPolicy, \
@@ -28,27 +28,30 @@ from paco.models.applications import Application, ResourceGroups, ResourceGroup,
     LambdaFunctionCode, LambdaVariable, SNSTopic, SNSTopicSubscription, \
     CloudFront, CloudFrontFactory, CloudFrontCustomErrorResponse, CloudFrontOrigin, CloudFrontCustomOriginConfig, \
     CloudFrontDefaultCacheBehavior, CloudFrontCacheBehavior, CloudFrontForwardedValues, CloudFrontCookies, CloudFrontViewerCertificate, \
-    RDSMysql, RDSPostgresql, ElastiCacheRedis, RDSOptionConfiguration, \
+    RDS, RDSMysql, RDSMysqlAurora, RDSPostgresql, RDSPostgresqlAurora, RDSDBClusterEventNotifications, RDSDBInstanceEventNotifications, \
+    RDSOptionConfiguration, RDSClusterInstance, RDSClusterInstances, RDSClusterDefaultInstance, \
     DeploymentPipeline, DeploymentPipelineConfiguration, DeploymentPipelineSourceStage, DeploymentPipelineBuildStage, \
     DeploymentPipelineDeployStage, DeploymentPipelineSourceCodeCommit, DeploymentPipelineBuildCodeBuild, \
     DeploymentPipelineDeployCodeDeploy, DeploymentPipelineManualApproval, CodeDeployMinimumHealthyHosts, \
-    DeploymentPipelineDeployS3, DeploymentPipelineLambdaInvoke, DeploymentPipelineSourceGitHub, \
+    DeploymentPipelineDeployS3, DeploymentPipelineLambdaInvoke, DeploymentPipelineSourceGitHub, DeploymentPipelineSourceECR, \
     DeploymentPipelinePacoCreateThenDeployImage, DeploymentPipelineDeployECS, CodePipelineStage, CodePipelineStages, \
     EFS, EFSMount, ASGScalingPolicies, ASGScalingPolicy, ASGLifecycleHooks, ASGLifecycleHook, ASGRollingUpdatePolicy, EIP, \
     EBS, EBSVolumeMount, SecretsManager, SecretsManagerApplication, SecretsManagerGroup, SecretsManagerSecret, \
-    GenerateSecretString, EC2LaunchOptions, DBParameterGroup, DBParameters, BlockDeviceMapping, BlockDevice, \
+    GenerateSecretString, EC2LaunchOptions, BlockDeviceMapping, BlockDevice, \
+    DBParameterGroup, DBClusterParameterGroup, DBParameters, \
     CodeDeployApplication, CodeDeployDeploymentGroups, CodeDeployDeploymentGroup, DeploymentGroupS3Location, \
     ElasticsearchDomain, ElasticsearchCluster, EBSOptions, ESAdvancedOptions, \
     ECSContainerDefinition, ECSContainerDefinitions, ECSTaskDefinitions, ECSTaskDefinition, \
     ECSLoadBalancer, ECSServicesContainer, ECSService, ECSCluster, ECSServices, PortMapping, ECSMountPoint, \
     ECSVolumesFrom, ECSVolume, ECSLogging, ECRRepository, ECSTaskDefinitionSecret, ECSContainerDependency, \
-    DockerLabels, ECSHostEntry, ECSHealthCheck, ECSUlimit
+    DockerLabels, ECSHostEntry, ECSHealthCheck, ECSUlimit, ECSCapacityProvider
 from paco.models.iot import IoTTopicRule, IoTTopicRuleAction, IoTTopicRuleLambdaAction, \
     IoTTopicRuleIoTAnalyticsAction, IoTAnalyticsPipeline, IoTPipelineActivities, IoTPipelineActivity, \
     IotAnalyticsStorage, Attributes, IoTDatasets, IoTDataset, DatasetTrigger, DatasetContentDeliveryRules, \
     DatasetContentDeliveryRule, DatasetS3Destination, DatasetQueryAction, DatasetContainerAction, \
     DatasetVariables, DatasetVariable, IoTPolicy, IoTVariables
-from paco.models.resources import EC2Resource, EC2KeyPairs, EC2KeyPair, S3Resource, S3Buckets, \
+from paco.models.resources import S3Resource, S3Buckets, \
+    EC2Resource, EC2KeyPairs, EC2KeyPair, EC2Users, EC2User, EC2Group, EC2Groups, \
     Route53Resource, Route53HostedZone, Route53RecordSet, Route53HostedZoneExternalResource, Route53HealthCheck, \
     CodeCommit, CodeCommitRepository, CodeCommitRepositoryGroup, CodeCommitUser, CodeCommitUsers, \
     CloudTrailResource, CloudTrails, CloudTrail, \
@@ -74,8 +77,8 @@ from paco.models.iam import IAM, ManagedPolicy, Role, Policy, AssumeRolePolicy,
 from paco.models.base import get_all_fields, most_specialized_interfaces, NameValuePair, RegionContainer, AccountRegions
 from paco.models.accounts import Account, AdminIAMUser
 from paco.models.references import Reference, PacoReference, FileReference
+from paco.models.references import resolve_ref, is_ref, get_model_obj_from_ref
 from paco.models.vocabulary import aws_regions
-from paco.models.references import resolve_ref, is_ref
 from paco.models.yaml import ModelYAML
 from paco.models import schemas
 from pathlib import Path
@@ -109,6 +112,7 @@ logger = get_logger()
 
 DEPLOYMENT_PIPELINE_STAGE_ACTION_CLASS_MAP = {
     'CodeCommit.Source': DeploymentPipelineSourceCodeCommit,
+    'ECR.Source': DeploymentPipelineSourceECR,
     'GitHub.Source': DeploymentPipelineSourceGitHub,
     'CodeBuild.Build': DeploymentPipelineBuildCodeBuild,
     'ManualApproval': DeploymentPipelineManualApproval,
@@ -131,6 +135,7 @@ RESOURCES_CLASS_MAP = {
     'ApiGatewayRestApi': ApiGatewayRestApi,
     'ASG': ASG,
     'DBParameterGroup': DBParameterGroup,
+    'DBClusterParameterGroup': DBClusterParameterGroup,
     'DeploymentPipeline': DeploymentPipeline,
     'EC2': EC2,
     'CloudFront': CloudFront,
@@ -154,7 +159,9 @@ RESOURCES_CLASS_MAP = {
     'ManagedPolicy': ManagedPolicy,
     'RDS': RDS,
     'RDSMysql': RDSMysql,
+    'RDSMysqlAurora': RDSMysqlAurora,
     'RDSPostgresql': RDSPostgresql,
+    'RDSPostgresqlAurora': RDSPostgresqlAurora,
     'Route53HealthCheck': Route53HealthCheck,
     'S3Bucket': ApplicationS3Bucket,
     'SNSTopic': SNSTopic,
@@ -233,8 +240,9 @@ SUB_TYPES_CLASS_MAP = {
     },
     EC2Resource: {
         'keypairs': ('container', (EC2KeyPairs, EC2KeyPair)),
+        'users': ('container', (EC2Users, EC2User)),
+        'groups': ('container', (EC2Groups, EC2Group)),
     },
-
     CloudWatchDashboard: {
         'variables': ('dynamic_dict', DashboardVariables),
     },
@@ -303,9 +311,13 @@ SUB_TYPES_CLASS_MAP = {
     DBParameterGroup: {
         'parameters': ('dynamic_dict', DBParameters)
     },
+    DBClusterParameterGroup: {
+        'parameters': ('dynamic_dict', DBParameters)
+    },
     DeploymentPipelineBuildCodeBuild: {
         'role_policies': ('obj_list', Policy),
         'codecommit_repo_users': ('str_list', PacoReference),
+        'secrets': ('str_list', PacoReference),
     },
     DeploymentPipelineDeployCodeDeploy: {
         'minimum_healthy_hosts': ('direct_obj', CodeDeployMinimumHealthyHosts)
@@ -334,7 +346,29 @@ SUB_TYPES_CLASS_MAP = {
         'response_models': ('obj_list', ApiGatewayMethodMethodResponseModel)
     },
     RDSOptionConfiguration: {
-        'option_settings': ('obj_list', NameValuePair)
+        'option_settings': ('obj_list', NameValuePair),
+    },
+    RDSMysqlAurora: {
+        'cluster_event_notifications': ('direct_obj', RDSDBClusterEventNotifications),
+        'db_instances': ('container', (RDSClusterInstances, RDSClusterInstance)),
+        'default_instance': ('direct_obj', RDSClusterInstance),
+        'dns': ('obj_list', DNS),
+        'read_dns': ('obj_list', DNS),
+    },
+    RDSPostgresqlAurora: {
+        'cluster_event_notifications': ('direct_obj', RDSDBClusterEventNotifications),
+        'db_instances': ('container', (RDSClusterInstances, RDSClusterInstance)),
+        'default_instance': ('direct_obj', RDSClusterDefaultInstance),
+        'dns': ('obj_list', DNS),
+        'read_dns': ('obj_list', DNS),
+    },
+    RDSClusterInstance: {
+        'event_notifications': ('direct_obj', RDSDBInstanceEventNotifications),
+        'monitoring': ('direct_obj', MonitorConfig)
+    },
+    RDSClusterDefaultInstance: {
+        'event_notifications': ('direct_obj', RDSDBInstanceEventNotifications),
+        'monitoring': ('direct_obj', MonitorConfig)
     },
     RDSMysql: {
         'option_configurations': ('obj_list', RDSOptionConfiguration),
@@ -430,6 +464,10 @@ SUB_TYPES_CLASS_MAP = {
         'block_device_mappings': ('obj_list', BlockDeviceMapping),
         'rolling_update_policy': ('direct_obj', ASGRollingUpdatePolicy),
         'ecs': ('direct_obj', ECSASGConfiguration),
+        'ssh_access': ('direct_obj', SSHAccess),
+    },
+    ECSASGConfiguration: {
+        'capacity_provider': ('direct_obj', ECSCapacityProvider),
     },
     EC2LaunchOptions: {
         'cfn_init_config_sets': ('str_list', zope.schema.TextLine)
@@ -880,7 +918,7 @@ Verify that '{}' has the correct indentation in the config file.
     if schemas.ICredentials.providedBy(obj):
         return
     for interface in most_specialized_interfaces(obj):
-        # invariants (these are also validate if they are explicitly part of a schema.Object() field)
+        # invariants (these are also to validate if they are explicitly part of a schema.Object() field)
         try:
             interface.validateInvariants(obj)
         except zope.interface.exceptions.Invalid as exc:
@@ -889,9 +927,10 @@ Verify that '{}' has the correct indentation in the config file.
         # validate all fields - this catches validation for required fields
         fields = zope.schema.getFields(interface)
         for name, field in fields.items():
+            value = getattr(obj, name, None)
             try:
                 if not field.readonly:
-                    field.validate(getattr(obj, name, None))
+                    field.validate(value)
             except ValidationError as exc:
                 raise_invalid_schema_error(obj, name, value, read_file_path, exc)
 
@@ -1287,8 +1326,8 @@ class ModelLoader():
         self.config_subdirs = {
             "monitor": self.instantiate_monitor_config,
             "accounts": self.instantiate_accounts,
-            "netenv": self.instantiate_network_environments,
             "resource": self.instantiate_resources,
+            "netenv": self.instantiate_network_environments,
         }
         # Legacy directory names
         if os.path.isdir(self.config_folder / 'NetworkEnvironments'):
@@ -1340,7 +1379,21 @@ Duplicate key \"{}\" found on line {} at column {}.
         return self.read_yaml_file(path)
 
     def load_all(self):
-        'Basically does a LOAD "*",8,1'
+        """The loader starts here with a LOAD "*",8,1
+
+        Loader control flow:
+         - check paco project version
+         - read project.yaml first
+         - read .credentials
+         - load YAML for each sub-dir in this order:
+           * monitor
+           * accounts
+           * resource
+           * netenv
+         - load YAML from services sub-dir last
+         - add core monitoring metrics to the model
+         - validate that paco.refs resolve to objects with the correct schema
+        """
         # set-up the Troposphere YAML constructors
         # ToDo: YAML constructors appear to be global (set on the SafeConstructor class)
         # This approach replaces Paco CFN constructors with ones that load CFN Tags as
@@ -1370,6 +1423,8 @@ Duplicate key \"{}\" found on line {} at column {}.
                         instantiate_method(name, config, os.path.join(subdir, fname))
         self.instantiate_services()
         self.load_core_monitoring()
+        # not yet ready for prime-time
+        #self.validate_ref_schemas()
         self.yaml.restore_existing_constructors()
 
     def read_paco_project_version(self):
@@ -1396,23 +1451,46 @@ Duplicate key \"{}\" found on line {} at column {}.
                     version[0], version[1], paco_models_version
                 ))
 
-    def load_references(self):
-        """
-        Resolves all references
-        A reference is a string that refers to another value in the model. The original
-        reference string is stored as '_ref_<attribute>' while the resolved reference is
-        stored in the attribute.
-        """
-        # walk the model
-        for model in get_all_nodes(self.project):
-            for name, field in get_all_fields(model).items():
-                # check for lists of refs
-                #if zope.schema.interfaces.IList.providedBy(field):
+    def validate_ref_schemas(self):
+        "Validate that every paco.ref resolves to an object that implements the schema_constraint for the ref"
+        for obj in get_all_nodes(self.project):
+            for name, field in get_all_fields(obj).items():
+                if schemas.IPacoReference.providedBy(field):
+                    value = getattr(obj, name, None)
+                    # paco.refs are already validated during attribute loading
+                    # if they are str_ok=True they do not need to be a ref and do not follow their schema_constraint
+                    if not is_ref(value):
+                        continue
+                    ref = Reference(value)
+                    if ref.type == 'function':
+                        continue
+                    # refs within the EnvironmentDefault do not get normalized and can't be resolved to a model obj
+                    env_default = get_parent_by_interface(obj, schemas.IEnvironmentDefault)
+                    if not schemas.IEnvironmentRegion.providedBy(env_default):
+                        continue
 
-                # check for refs as normal attrs
-                if hasattr(model, '_ref_' + name):
-                    value = resolve_ref(getattr(model, '_ref_' + name), self.project)
-                    setattr(model, name, value)
+                    ref_obj = get_model_obj_from_ref(value, self.project, obj)
+                    # make the ref obj available in the model
+                    # ToDo: obj.get_ref_obj(fieldname) could replace get_model_obj_from_ref?
+                    setattr(obj, '_ref_' + name, ref_obj)
+                    if isinstance(field.schema_constraint, str):
+                        # can resolve to any kind of obj
+                        if field.schema_constraint == 'Interface':
+                            continue
+                        # not every PacoReference declares a schema_constraint
+                        # every paco.models.schemas should, but paco add-ons may not
+                        if field.schema_constraint == '':
+                            continue
+                        schema = getattr(schemas, field.schema_constraint)
+                    else:
+                        schema = field.schema_constraint
+                    if not schema.providedBy(ref_obj):
+                        message = "\nA paco.ref is refering to the wrong type of configuration:\n\n"
+                        message += f"Object: {obj.paco_ref_parts}\n"
+                        message += f"Field: {name}\n"
+                        message += f"Ref: {value}\n\n"
+                        message += f"Expected configuration of type '{field.schema_constraint}' but got 'I{ref_obj.__class__.__name__}'\n"
+                        raise InvalidPacoReference(message)
 
     def load_core_monitoring(self):
         "Loads monitoring metrics that are built into resources"
@@ -1481,11 +1559,11 @@ Duplicate key \"{}\" found on line {} at column {}.
             end_idx = len(paco_ref)
         str_idx = paco_ref.find("'", sub_idx, end_idx)
         if str_idx == -1:
-            raise StackException(PacoErrorCode.Unknown)
+            raise InvalidPacoSub(f"Invalid paco.sub: {paco_ref}\nCould not find single quote after paco.sub.")
         str_idx += 1
         end_str_idx = paco_ref.find("'", str_idx, end_idx)
         if end_str_idx == -1:
-            raise StackException(PacoErrorCode.Unknown)
+            raise InvalidPacoSub(f"Invalid paco.sub: {paco_ref}")
 
         # Isolate any ${} replacements
         first_pass = True
@@ -1493,7 +1571,7 @@ Duplicate key \"{}\" found on line {} at column {}.
             dollar_idx = paco_ref.find("${", str_idx, end_str_idx)
             if dollar_idx == -1:
                 if first_pass == True:
-                    raise StackException(PacoErrorCode.Unknown)
+                    raise InvalidPacoSub(f"Invalid paco.sub: {paco_ref}")
                 else:
                     break
             rep_1_idx = dollar_idx
@@ -1501,6 +1579,7 @@ Duplicate key \"{}\" found on line {} at column {}.
             netenv_ref_idx = paco_ref.find(
                 "paco.ref netenv.", rep_1_idx, rep_2_idx)
             if netenv_ref_idx != -1:
+
                 sub_ref_idx = netenv_ref_idx
                 sub_ref_end_idx = sub_ref_idx+(rep_2_idx-sub_ref_idx-1)
                 sub_ref = paco_ref[sub_ref_idx:sub_ref_end_idx]
@@ -1550,6 +1629,11 @@ Duplicate key \"{}\" found on line {} at column {}.
 
                     ref.parts[3] = application.name
 
+        # Ignore if the environment and region are already specified
+        if ref.parts[2] in self.project['netenv'][ref.parts[1]].keys():
+            if ref.parts[3] in self.project['netenv'][ref.parts[1]][ref.parts[2]].keys():
+                return paco_ref
+
         ref.parts.insert(2, env_id)
         ref.parts.insert(3, region)
         new_ref_parts = '.'.join(ref.parts)
