@@ -254,6 +254,24 @@ class EC2KeyPair(Named):
     account = FieldProperty(schemas.IEC2KeyPair['account'])
     keypair_name = FieldProperty(schemas.IEC2KeyPair['keypair_name'])
 
+@implementer(schemas.IEC2Users)
+class EC2Users(Named, dict):
+    pass
+
+@implementer(schemas.IEC2User)
+class EC2User(Named):
+    full_name = FieldProperty(schemas.IEC2User['full_name'])
+    email = FieldProperty(schemas.IEC2User['email'])
+    public_ssh_key = FieldProperty(schemas.IEC2User['public_ssh_key'])
+
+@implementer(schemas.IEC2Groups)
+class EC2Groups(Named, dict):
+    pass
+
+@implementer(schemas.IEC2Group)
+class EC2Group(Named):
+    members = FieldProperty(schemas.IEC2Group['members'])
+
 @implementer(schemas.IEC2Resource)
 class EC2Resource(Named):
     keypairs = FieldProperty(schemas.IEC2Resource['keypairs'])
@@ -309,6 +327,8 @@ class Route53HostedZone(Named, Deployable):
     record_sets = FieldProperty(schemas.IRoute53HostedZone["record_sets"])
     parent_zone = FieldProperty(schemas.IRoute53HostedZone["parent_zone"])
     external_resource = FieldProperty(schemas.IRoute53HostedZone["external_resource"])
+    private_hosted_zone = FieldProperty(schemas.IRoute53HostedZone["private_hosted_zone"])
+    vpc_associations = FieldProperty(schemas.IRoute53HostedZone["vpc_associations"])
 
     def __init__(self, name, parent):
         super().__init__(name, parent)
