@@ -20,6 +20,14 @@ network_protocols = SimpleVocabulary.fromValues([
   'udp',
 ])
 
+log_levels = SimpleVocabulary.fromValues([
+	'crit',
+	'error',
+	'warn',
+	'info',
+	'debug',
+])
+
 ecs_network_modes = SimpleVocabulary.fromValues([
 	'awsvpc',
 	'bridge',
@@ -103,6 +111,45 @@ application_group_types = [
     'Deployment',
 ]
 
+rds_cluster_engine_mode = SimpleVocabulary.fromValues([
+	'provisioned',
+	'serverless',
+	'parallelquery',
+	'global',
+	'multimaster'
+])
+
+rds_restore_types = SimpleVocabulary.fromValues([
+	'full-copy',
+	'copy-on-write',
+])
+
+rds_cluster_event_categories = SimpleVocabulary.fromValues([
+	"maintenance",
+	"failover",
+	"notification",
+	"creation",
+	"failure",
+	"deletion",
+])
+
+rds_instance_event_categories = SimpleVocabulary.fromValues([
+	"backup",
+	"maintenance",
+	"failover",
+	"creation",
+	"configuration change",
+	"recovery",
+	"deletion",
+	"restoration",
+	"failure",
+	"read replica",
+	"availability",
+	"low storage",
+	"notification",
+	"backtrack",
+])
+
 cloudwatch = {
 	'App': {
 		'dimension': '',
@@ -121,18 +168,18 @@ cloudwatch = {
 		'dimension': 'CacheClusterId',
 		'namespace': 'AWS/ElastiCache'
 	},
-  'ElasticsearchDomain': {
-    'dimension': 'DomainName',
-    'namespace': 'AWS/ES',
-  },
-  'IoTTopicRule': {
-      'dimension': 'RuleName',
-      'namespace': 'AWS/IoT',
-  },
-  'IoTAnalyticsPipeline': {
-      'dimension': '',
-      'namespace': 'AWS/IoTAnalytics',
-  },
+	'ElasticsearchDomain': {
+		'dimension': 'DomainName',
+		'namespace': 'AWS/ES',
+	},
+	'IoTTopicRule': {
+		'dimension': 'RuleName',
+		'namespace': 'AWS/IoT',
+	},
+	'IoTAnalyticsPipeline': {
+		'dimension': '',
+		'namespace': 'AWS/IoTAnalytics',
+	},
 	'Lambda': {
 		'dimension': 'FunctionName',
 		'namespace': 'AWS/Lambda'
@@ -145,10 +192,18 @@ cloudwatch = {
 		'dimension': 'DBInstanceIdentifier',
 		'namespace': 'AWS/RDS'
 	},
+	'RDSMysqlAurora': {
+		'dimension': 'DBInstanceIdentifier',
+		'namespace': 'AWS/RDS'
+	},
 	'RDSPostgresql': {
 		'dimension': 'DBInstanceIdentifier',
 		'namespace': 'AWS/RDS'
 	},
+	'RDSPostgresqlAurora': {
+		'dimension': 'DBInstanceIdentifier',
+		'namespace': 'AWS/RDS'
+	},
     'IoTTopicRule': {
         'dimension': 'RuleName',
         'namespace': 'AWS/IoT',
